@@ -43,10 +43,6 @@ class CLIArgs:  # pylint: disable=R0902,R0903
         The notebooks or directories to run third-party tool on.
     nbqa_mutate
         Whether to allow 3rd party tools to modify notebooks.
-    nbqa_config
-        Config file for 3rd party tool (e.g. :code:`.mypy.ini`)
-    nbqa_ignore_cells
-        Deprecated.
     nbqa_process_cells
         Process code within these cell magics.
     nbqa_addopts
@@ -61,9 +57,7 @@ class CLIArgs:  # pylint: disable=R0902,R0903
     root_dirs: Sequence[str]
     nbqa_addopts: Sequence[str]
     nbqa_mutate: bool
-    nbqa_ignore_cells: Optional[str]
     nbqa_process_cells: Optional[str]
-    nbqa_config: Optional[str]
     nbqa_diff: bool
     nbqa_files: Optional[str]
     nbqa_exclude: Optional[str]
@@ -83,8 +77,6 @@ class CLIArgs:  # pylint: disable=R0902,R0903
         self.root_dirs = args.root_dirs
         self.nbqa_addopts = cmd_args
         self.nbqa_mutate = args.nbqa_mutate or False
-        self.nbqa_config = args.nbqa_config or None
-        self.nbqa_ignore_cells = args.nbqa_ignore_cells or None
         self.nbqa_process_cells = args.nbqa_process_cells or None
         self.nbqa_diff = args.nbqa_diff or False
         self.nbqa_files = args.nbqa_files or None
@@ -130,20 +122,6 @@ class CLIArgs:  # pylint: disable=R0902,R0903
             action="store_true",
             help="Show diff which would result from running --nbqa-mutate.",
         )
-        parser.add_argument(
-            "--nbqa-config",
-            required=False,
-            help="Config file for third-party tool (e.g. `setup.cfg`)",
-        )
-        parser.add_argument(
-            "--nbqa-ignore-cells",
-            required=False,
-            help=dedent(
-                r"""
-                Deprecated, do not use.
-                """
-            ),
-        )
         parser.add_argument(
             "--nbqa-process-cells",
             required=False,
